@@ -79,7 +79,7 @@ PROGRAM ButterflyPACK_IE_2D
 	character(len=10) which
     integer ido, n, nx, nev, ncv, lworkl, info, nconv, maxitr, ishfts, mode
     complex(kind=8) sigma
-    real(kind=8) tol
+    real(kind=8):: tol, shift_r=0d0, shift_i=0d0
     logical rvec
 	real(kind=8),external :: pdznorm2, dlapy2
 	integer nargs,flag
@@ -174,7 +174,11 @@ PROGRAM ButterflyPACK_IE_2D
 						else if	(trim(strings)=='--nev')then
 							read(strings1,*)quant%nev
 						else if	(trim(strings)=='--tol_eig')then
-							read(strings1,*)quant%tol_eig
+							read(strings1,*)quant%tol_eig						
+						else if	(trim(strings)=='--shift_r')then
+							read(strings1,*)shift_r
+						else if	(trim(strings)=='--shift_i')then
+							read(strings1,*)shift_i					
 						else if	(trim(strings)=='--which')then
 							quant%which=trim(strings1)
 						else
@@ -195,6 +199,8 @@ PROGRAM ButterflyPACK_IE_2D
 		endif
 	enddo
 
+	quant%shift = shift_r + shift_i*BPACK_junit
+
 	call z_PrintOptions(option_A,ptree_A)
 
     quant%wavenum=2*BPACK_pi/quant%wavelength
